/*************************************************************************
TROUBLESHOOT -- Circuit troubleshooting simulator program
By Tony R. Kuphaldt
Last update 10 February 2021

This software is released under the CC0 1.0 Universal license,
which is equivalent to Public Domain.

   [Circuit 002] -- three-resistor current divider with DC current source

   Three resistors connected in parallel, controlled by a switch and 
   powered by one current source

*************************************************************************/

#include "tshoot.h"		// Contains all the declarations specific to tshoot
#include <stdio.h>
#include <stdlib.h>		// Necessary for the "random" function to work
#include <time.h>		// Necessary for the "time" library functions to work
#include <math.h>

int init_002 (void);
int fault_002 (void);
int sim_002 (void);

int
circuit_002 (void)
{
  clear_screen ();

  init_002 ();			// Initialize component values
  fault_002 ();			// Randomly choose a fault
  sim_002 ();			// Simulate circuit

  // Get the current UNIX system time as the timestamp that the user
  // began troubleshooting
  steplog[0].time = (int) (time (NULL));

  while (mistake == 0 && conclusion == -1)	// Loop runs until a mistake or conclusion is made
    {
      print_components (1);	// 1 = Print component nominal values and switch/jumper statuses

      print_measurements ();

      user_test ();

      sim_002 ();

      clear_screen ();

    }

  return 0;
}




int
init_002 (void)
{

  int n;

  tp = 5;			// 5 test points in circuit in addition to TP 0

  // Here we use the "health" integer variable of element 0 for each type
  // of circuit component as a count for how many of these components will
  // be in the circuit, since no circuit ever contains a "component 0" 
  // (e.g. resistor R0 or switch S0).  For jumpers we use the "status"
  // element (i.e. j[0].s).
  //
  // Following the "blank_slate()" function, all health and status values
  // should be set to zero, so all we must do here is specify components 
  // that are in the circuit (i.e. we don't have to declare what IS NOT in 
  // the circuit, but only what IS).
  dci[0].h = 1;
  r[0].h = 3;
  s[0].h = 1;

  dci[1].selected = dci[1].alt[random_whole (9)];	// Randomly selecting nominal source current value
  dci[1].i[0] = random_component (dci[1].selected, dci[1].tolerance);	// Setting actual DC source current 

  for (n = 1; n <= 3; ++n)	// Randomizing resistor values for R1 through R3
    {
      r[n].selected = r[n].alt[random_whole (9)];
      r[n].r[0] = random_component (r[n].selected, r[n].tolerance);
    }

//  s[1].s = random_whole (1);  // Toggle switch state set randomly
  s[1].s = 1;			// Toggle switch begins in the "on" position

  // Jumper wires ready to connect between pre-defined test points
  // j[1].jtp[0] = 3;
  // j[1].jtp[1] = 4;
  // j[2].jtp[0] = 2;
  // j[2].jtp[1] = 5;

  vm_red = 5;			// Voltmeter red lead on test point 5
  vm_black = 0;			// Voltmeter black lead on test point 0
  vm_cost = 2;			// Voltmeter costs $2 to move test leads

  am_cost = 5;			// Ammeter costs $5 to move 

  j_cost = 10;			// Inserting a temporary jumper wire costs $10

  noise = 0.05;			// Meter noise = +/- 0.05%

  return 0;
}




int
fault_002 (void)
{
  int n;
  int max = 9;			// Number of possible faults (excluding 0 = No fault)

  // Initialize fault array
  faults[0] = "No fault";
  faults[1] = "Dead source";
  faults[2] = "Switch failed open";
  faults[3] = "Switch failed shorted";
  faults[4] = "Resistor R1 failed open";
  faults[5] = "Resistor R1 failed shorted";
  faults[6] = "Resistor R2 failed open";
  faults[7] = "Resistor R2 failed shorted";
  faults[8] = "Resistor R3 failed open";
  faults[9] = "Resistor R3 failed shorted";

  printf
    ("If you haven't already, please open the schematic diagram file \n");
  printf ("`circuit_002.pdf' for reference while troubleshooting \n \n");

  printf
    ("The computer will now randomly select one condition from the following list: \n \n");

  for (n = 0; n <= max; ++n)
    printf ("Condition (%i) = %s \n", n, faults[n]);

  printf
    ("\nYour task will be to determine which one of these conditions exists \n");
  printf
    ("based on virtual tests you will perform on the circuit such as \n");
  printf
    ("toggling switches and placing meters at various test points. \n \n");

  printf ("The troubleshooting scenario will begin with the voltmeter \n");
  printf ("connected between test points TP%i and TP%i and switch S1 \n",
	  vm_red, vm_black);
  printf ("in the `on' state. \n \n");

  printf ("Please type your name (no spaces, 40 characters maximum) \n");
  printf ("and press the <Enter> key to continue \n");

  scanf ("%40s", username);

  // Discard characters from stdin buffer until a linefeed (LF, or ASCII code 10)
  // is detected.
  while (getchar () != 10)
    {
      // Does nothing!
    }

  clear_screen ();

  fault = random_whole (max);

  // If command-line option "-nofault" or "-fault N" invoked, we override any randomly-chosen
  // fault and set the fault code to the override value!
  if (fault_override >= 0)
    fault = fault_override;

  // Default "par" scores
  par_steps = 4;
  par_cost = 11;
  par_time = 60;		// 60 seconds = 1 minute

  switch (fault)
    {
    case 1:			// Source I1 dead
      dci[1].h = 1;
      par_steps = 3;
      par_cost = 7;
      break;

    case 2:			// Switch S1 failed open
      s[1].h = 1;
      par_steps = 1;
      par_cost = 1;
      break;

    case 3:			// Switch S1 failed shorted
      s[1].h = 2;
      par_steps = 6;
      par_cost = 25;
      break;

    case 4:			// Resistor R1 failed open
      r[1].h = 1;
      par_steps = 4;
      par_cost = 11;
      break;

    case 5:			// Resistor R1 failed shorted
      r[1].h = 2;
      par_steps = 6;
      par_cost = 25;
      break;

    case 6:			// Resistor R2 failed open
      r[2].h = 1;
      par_steps = 4;
      par_cost = 11;
      break;

    case 7:			// Resistor R2 failed shorted
      r[2].h = 2;
      par_steps = 6;
      par_cost = 25;
      break;

    case 8:			// Resistor R3 failed open
      r[3].h = 1;
      par_steps = 4;
      par_cost = 11;
      break;

    case 9:			// Resistor R3 failed shorted
      r[3].h = 2;
      par_steps = 6;
      par_cost = 25;
      break;
    }

  // Calculating "limits" for steps taken, testing budget, and time
  if (fault_override == -1)
    {
      limit_steps = 4 * par_steps;
      limit_cost = 4 * par_cost;
      limit_time = 4 * par_time;
    }

  else
    {
      limit_steps = 999999;
      limit_cost = 999999;
      limit_time = 999999;
    }

  return 0;
}





int
sim_002 (void)
{
  int n, m;

  // Here we call the update_pots() function to update the resistance
  // value for each portion of each potentiometer, based on total
  // resistance (r_total) and wiper position (x).
  update_pots ();

  // Declare shorthand variables
  float r_total, r1, r2, r3, r_switch;
  float i_src, v_src;

  // Initialize shorthand variables
  r1 = r[1].r[r[1].h];
  r2 = r[2].r[r[2].h];
  r3 = r[3].r[r[3].h];
  r_switch = s[1].r[s[1].s][s[1].h];
  i_src = dci[1].i[dci[1].h];

  // Begin circuit analysis
  r_total = 1 / ((1 / r1) + (1 / r2) + (1 / r3) + (1 / r_switch));
  v_src = i_src * r_total;

  // Set test point 0 and 5 currents to the source current value
  i_tp[0] = i_src;
  i_tp[5] = i_src;

  // Calculate branch test point currents using Ohm's Law
  i_tp[1] = v_src / r1;
  i_tp[2] = v_src / r2;
  i_tp[3] = v_src / r3;
  i_tp[4] = v_src / r_switch;

  // Calculate all test point voltages with respect to ground (TP 0)
  v_tp[0][0] = 0.0;		// TP 0 (ground) is 0 Volts by definition . . .
  v_tp[1][0] = 0.0;		// These test points are electrically common to each other
  v_tp[2][0] = 0.0;		// These test points are electrically common to each other
  v_tp[3][0] = 0.0;		// These test points are electrically common to each other
  v_tp[4][0] = 0.0;		// These test points are electrically common to each other
  v_tp[5][0] = v_src;

  // Calculate all test point-pair voltages
  for (n = 0; n < COUNT; ++n)
    {
      for (m = 0; m < COUNT; ++m)
	v_tp[n][m] = v_tp[n][0] - v_tp[m][0];
    }


  return 0;
}
