/*************************************************************************
TROUBLESHOOT -- Circuit troubleshooting simulator program
By Tony R. Kuphaldt
Last update 10 February 2021

This software is released under the CC0 1.0 Universal license,
which is equivalent to Public Domain.

   [Circuit 009] -- relay-controlled motor circuit

   One small DC electric motor turned on and off by a
   combinational logic circuit comprised of TTL 
   semiconductor logic gates

*************************************************************************/

#include "tshoot.h"		// Contains all the declarations specific to tshoot
#include <stdio.h>
#include <stdlib.h>		// Necessary for the "random" function to work
#include <time.h>		// Necessary for the "time" library functions to work
#include <math.h>

int init_009 (void);
int fault_009 (void);
int sim_009 (void);

int
circuit_009 (void)
{
  clear_screen ();

  init_009 ();			// Initialize component values
  fault_009 ();			// Randomly choose a fault
  sim_009 ();			// Simulate circuit

  // Get the current UNIX system time as the timestamp that the user
  // began troubleshooting
  steplog[0].time = (int) (time (NULL));

  while (mistake == 0 && conclusion == -1)	// Loop runs until a mistake or conclusion is made
    {
      print_components (1);	// 1 = Print component nominal values and switch/jumper statuses

      print_measurements ();

      user_test ();

      sim_009 ();

      clear_screen ();

    }

  return 0;
}




int
init_009 (void)
{

  tp = 8;			// 8 test points in circuit in addition to TP 0

  // Here we use the "health" integer variable of element 0 for each type
  // of circuit component as a count for how many of these components will
  // be in the circuit, since no circuit ever contains a "component 0" 
  // (e.g. resistor R0 or switch S0).  For jumpers we use the "status"
  // element (i.e. j[0].s).
  //
  // Following the "blank_slate()" function, all health and status values
  // should be set to zero, so all we must do here is specify components 
  // that are in the circuit (i.e. we don't have to declare what IS NOT in 
  // the circuit, but only what IS).
  dcv[0].h = 2;
  r[0].h = 1;			// Representing the relay coil as a resistor
  ld[0].h = 1;			// Representing the motor as a generic load
  s[0].h = 2;			// Two switches for digital logic input states
  rc[0].h = 5;			// Five relays (3 gates, 1 MOSFET, 1 actual relay)

  dcv[1].alt[0] = 22.0;		// Establishing new nominal DC voltage source values
  dcv[1].alt[1] = 22.5;		// appropriate for powering an industrial relay and
  dcv[1].alt[2] = 23.0;		// a DC motor.
  dcv[1].alt[3] = 23.5;
  dcv[1].alt[4] = 24.0;
  dcv[1].alt[5] = 24.5;
  dcv[1].alt[6] = 25.0;
  dcv[1].alt[7] = 25.5;
  dcv[1].alt[8] = 26.0;
  dcv[1].alt[9] = 26.5;

  dcv[1].tolerance = random_component (5.0, 75.0);	// Randomly sets the source's tolerance between +/-1.25% and +/-8.75%
  dcv[1].selected = dcv[1].alt[random_whole (9)];	// Randomly selecting nominal source voltage value
  dcv[1].v[0] = random_component (dcv[1].selected, dcv[1].tolerance);	// Setting actual DC source voltage

  dcv[2].alt[0] = 5.0;		// Establishing new nominal DC voltage source values
  dcv[2].alt[1] = 5.0;		// appropriate for powering TTL logic
  dcv[2].alt[2] = 5.0;
  dcv[2].alt[3] = 5.0;
  dcv[2].alt[4] = 5.0;
  dcv[2].alt[5] = 5.0;
  dcv[2].alt[6] = 5.0;
  dcv[2].alt[7] = 5.0;
  dcv[2].alt[8] = 5.0;
  dcv[2].alt[9] = 5.0;

  dcv[2].tolerance = random_component (2.0, 10.0);	// Randomly sets the source's tolerance between +/-1.8% and +/-2.2%
  dcv[2].selected = dcv[2].alt[random_whole (9)];	// Randomly selecting nominal source voltage value
  dcv[2].v[0] = random_component (dcv[2].selected, dcv[2].tolerance);	// Setting actual DC source voltage

  r[1].tolerance = random_component (10.0, 50.0);	// Randomly sets the relay coil resistance tolerance between +/-5% and +/-15%
  r[1].selected = dcv[1].selected / 0.100;	// Selecting relay coil resistance based on the DC voltage
  // source nominal value and a nominal 100 mA coil current
  r[1].r[0] = random_component (r[1].selected, r[1].tolerance);	// Setting actual relay coil resistance value

  // Gate outputs and MOSFET channel modeled as relay switch contacts
  rc[1].r[1][0] = random_component (500.0, 10.0);	// 500 Ohms +/- 10% gate sinking resistance
  rc[2].r[1][0] = random_component (500.0, 10.0);	// 500 Ohms +/- 10% gate sinking resistance
  rc[3].r[1][0] = random_component (500.0, 10.0);	// 500 Ohms +/- 10% gate sinking resistance
  rc[4].r[1][0] = random_component (100e-3, 30);	// 0.1 Ohm +/- 30% MOSFET channel "on" resistance
  rc[5].r[1][0] = random_component (1e-2, 10);	// 0.01 Ohm +/- 10% relay contact "on" resistance

  ld[1].d = "Motor";		// Giving our load a name (motor)
  ld[1].r[0] = 24.0 / 0.5;	// Selecting motor resistance based on an assumed curent of 0.5 Amperes at 24 VDC
  ld[1].v_min = 18.0;		// Selecting minimum "energized" motor voltage
  ld[1].i_min = 0.3;		// Selecting minimum "energized" motor current

  vm_red = 1;			// Voltmeter red lead on test point 8
  vm_black = 0;			// Voltmeter black lead on test point 9
  vm_cost = 2;			// Voltmeter costs $2 to move test leads

  am_cost = 5;			// Ammeter costs $5 to move 

  noise = 0.01;			// Meter noise = +/- 0.01%

  return 0;
}




int
fault_009 (void)
{
  int n;
  int max = 10;			// Number of possible faults (excluding 0 = No fault)

  // Initialize fault array
  faults[0] = "No fault";
  faults[1] = "Gate U1 output failed low";
  faults[2] = "Gate U1 output failed high";
  faults[3] = "Gate U2 output failed low";
  faults[4] = "Gate U2 output failed high";
  faults[5] = "Gate U3 output failed low";
  faults[6] = "Gate U3 output failed high";
  faults[7] = "Transistor Q1 failed open D-S";
  faults[8] = "Relay contact failed open";
  faults[9] = "Relay contact failed shorted";
  faults[10] = "Motor failed open";

  printf
    ("If you haven't already, please open the schematic diagram file \n");
  printf ("`circuit_009.pdf' for reference while troubleshooting \n \n");

  printf
    ("The computer will now randomly select one condition from the following list: \n \n");

  for (n = 0; n <= max; ++n)
    printf ("Condition (%i) = %s \n", n, faults[n]);

  printf
    ("\nYour task will be to determine which one of these conditions exists \n");
  printf
    ("based on virtual tests you will perform on the circuit such as \n");
  printf
    ("toggling switches and placing meters at various test points. \n \n");

  printf ("The troubleshooting scenario will begin with the voltmeter \n");
  printf ("connected between test points TP%i and TP%i. \n \n",
	  vm_red, vm_black);

  printf ("Please type your name (no spaces, 40 characters maximum) \n");
  printf ("and press the <Enter> key to continue \n");

  scanf ("%40s", username);

  // Discard characters from stdin buffer until a linefeed (LF, or ASCII code 10)
  // is detected.
  while (getchar () != 10)
    {
      // Does nothing!
    }

  clear_screen ();

  fault = random_whole (max);

  // If command-line option "-nofault" or "-fault N" invoked, we override any randomly-chosen
  // fault and set the fault code to the override value!
  if (fault_override >= 0)
    fault = fault_override;

  // Default "par" scores
  par_steps = 4;
  par_cost = 4;
  par_time = 60;		// 60 seconds = 1 minute

  switch (fault)
    {
    case 1:			// Gate U1 output failed low
      rc[1].h = 2;
      par_steps = 4;
      par_cost = 6;
      break;

    case 2:			// Gate U1 output failed high
      rc[1].h = 1;
      par_steps = 6;
      par_cost = 9;
      break;

    case 3:			// Gate U2 output failed low
      rc[2].h = 2;
      par_steps = 4;
      par_cost = 5;
      break;

    case 4:			// Gate U2 output failed high
      rc[2].h = 1;
      par_steps = 6;
      par_cost = 9;
      break;

    case 5:			// Gate U3 output failed low
      rc[3].h = 2;
      par_steps = 4;
      par_cost = 6;
      break;

    case 6:			// Gate U3 output failed high
      rc[3].h = 1;
      par_steps = 6;
      par_cost = 9;
      break;

    case 7:			// Transistor Q1 failed open D-S
      rc[4].h = 1;
      par_steps = 4;
      par_cost = 6;
      break;

    case 8:			// Relay contact failed open
      rc[5].h = 1;
      par_steps = 5;
      par_cost = 8;
      break;

    case 9:			// Relay contact failed shorted
      rc[5].h = 2;
      par_steps = 4;
      par_cost = 5;
      break;

    case 10:			// Motor failed open
      ld[1].h = 1;
      par_steps = 5;
      par_cost = 8;
      break;
    }

  // Calculating "limits" for steps taken, testing budget, and time
  if (fault_override == -1)
    {
      limit_steps = 4 * par_steps;
      limit_cost = 4 * par_cost;
      limit_time = 4 * par_time;
    }

  else
    {
      limit_steps = 999999;
      limit_cost = 999999;
      limit_time = 999999;
    }

  return 0;
}





int
sim_009 (void)
{
  int n, m;

  // NOTE: all logic gates as well as the MOSFET are modeled here
  // as "relays" since they operate in discrete on/off modes

  // Declare shorthand variables
  float v1_src, vcc;
  float v_u1a, v_u1b, v_u1out;	// Gate U1 input and output voltages
  float v_u2a, v_u2b, v_u2out;	// Gate U2 input and output voltages
  float v_u3a, v_u3b, v_u3out;	// Gate U2 input and output voltages
  float r_u1out, r_u2out, r_u3out;	// Open-collector resistances of gate outputs
  float r_pullup;		// TTL pullup resistor value inside gate
  float v_to = 2.0;		// Turn-on voltage of MOSFET
  float r_coil, r_motor, r_sw1, r_sw2, r_q1, r_contact;
  float i_coil, i_motor;
  float v_coil;

  // Initialize shorthand variables
  v1_src = dcv[1].v[dcv[1].h];	// Nominal 24 VDC source
  vcc = dcv[2].v[dcv[2].h];	// TTL logic source (V_CC)
  r_coil = r[1].r[r[1].h];
  r_motor = ld[1].r[ld[1].h];
  r_sw1 = s[1].r[s[1].s][s[1].h];
  r_sw2 = s[2].r[s[2].s][s[2].h];
  r_pullup = 10e3;		// Internal pullup resistor value for TTL gate input


  // Calculating U1 and U2 input voltages
  v_u1a = vcc * (r_sw1 / (r_sw1 + r_pullup));
  v_u1b = vcc * (r_sw2 / (r_sw2 + r_pullup));
  v_u2a = v_u1a;
  v_u2b = v_u1b;


  // Set U1 (AND gate) contact status based on standard TTL thresholds,
  // V_IL = 0.8 Volts and V_IH = 2.0 Volts
  if (v_u1a > 2.0 && v_u1b > 2.0)
    rc[1].s = 0;		// Open-collector output "switch" is open to make output float

  else
    rc[1].s = 1;		// Open-collector output "switch" is closed to pull output low

  r_u1out = rc[1].r[rc[1].s][rc[1].h];
  v_u1out = vcc * (r_u1out / (r_u1out + r_pullup));
  v_u3a = v_u1out;


  // Set U2 (Neg-AND gate) contact status based on standard TTL thresholds,
  // V_IL = 0.8 Volts and V_IH = 2.0 Volts
  if (v_u2a < 0.8 && v_u2b < 0.8)
    rc[2].s = 0;		// Open-collector output "switch" is open to make output float

  else
    rc[2].s = 1;		// Open-collector output "switch" is closed to pull output low

  r_u2out = rc[2].r[rc[2].s][rc[2].h];
  v_u2out = vcc * (r_u2out / (r_u2out + r_pullup));
  v_u3b = v_u2out;

  // Set U3 (OR gate) contact status based on standard TTL thresholds,
  // V_IL = 0.8 Volts and V_IH = 2.0 Volts
  if (v_u3a > 2.0 || v_u3b > 2.0)
    rc[3].s = 0;		// Open-collector output "switch" is open to make output float

  else
    rc[3].s = 1;		// Open-collector output "switch" is closed to pull output low

  r_u3out = rc[3].r[rc[3].s][rc[3].h];
  v_u3out = vcc * (r_u3out / (r_u3out + r_pullup));


  // Set Q1 contact status based on V_TO threshold
  if (v_u3out > v_to)
    rc[4].s = 1;		// Drain-Source "switch" is closed

  else
    rc[4].s = 0;		// Drain-Source "switch" is open

  r_q1 = rc[4].r[rc[4].s][rc[4].h];


  // Calculating relay coil parameters
  i_coil = v1_src / (r_coil + r_q1);
  v_coil = i_coil * r_coil;


  // Set relay contact status based on relay coil voltage and current;
  // current must exceed 50 mA and voltage must exceed half of v1_src
  if (i_coil > 0.05 && v_coil > (v1_src / 2))
    rc[5].s = 1;		// Closed contact

  else
    rc[5].s = 0;		// Open contact

  r_contact = rc[5].r[rc[5].s][rc[5].h];

  // Calculating load (motor) parameters
  i_motor = v1_src / (r_contact + r_motor);
  ld[1].i = i_motor;
  ld[1].v = i_motor * r_motor;

  // Set test point currents in motor circuit
  i_tp[8] = i_motor;
  i_tp[7] = i_coil + i_motor;
  i_tp[6] = i_coil;

  // Here we call the update_loads() function to update the 
  // energization statuses of all loads in the circuit, based
  // on load voltage and load current.
  update_loads ();

  // Calculate all test point voltages with respect to ground (TP 0)
  v_tp[0][0] = 0.0;		// TP 0 (ground) is 0 Volts by definition . . .
  v_tp[1][0] = v_u1a;
  v_tp[2][0] = v_u1b;
  v_tp[3][0] = v_u1out;
  v_tp[4][0] = v_u2out;
  v_tp[5][0] = v_u3out;
  v_tp[6][0] = i_coil * r_q1;
  v_tp[7][0] = v1_src;
  v_tp[8][0] = i_motor * r_motor;

  // Calculate all test point-pair voltages
  for (n = 0; n < COUNT; ++n)
    {
      for (m = 0; m < COUNT; ++m)
	v_tp[n][m] = v_tp[n][0] - v_tp[m][0];
    }


  if (r_contact > 1e6 && r_motor > 1e6)	// Relay contact and motor both open leaving TP8 floating
    {
      force_voltage (8, 0, 0.0);
      force_voltage (8, 1, 0.0);
      force_voltage (8, 2, 0.0);
      force_voltage (8, 3, 0.0);
      force_voltage (8, 4, 0.0);
      force_voltage (8, 5, 0.0);
      force_voltage (8, 6, 0.0);
      force_voltage (8, 7, 0.0);
    }






  return 0;
}
