/*************************************************************************
TROUBLESHOOT -- Circuit troubleshooting simulator program
By Tony R. Kuphaldt
Last update 10 February 2021

This software is released under the CC0 1.0 Universal license,
which is equivalent to Public Domain.

   [Circuit 010] -- RLC band-pass filter with signal generator

   Series resistor and parallel-resonant tank circuit, powered 
   powered by an adjustable-frequency AC voltage source

*************************************************************************/

#include "tshoot.h"		// Contains all the declarations specific to tshoot
#include <stdio.h>
#include <stdlib.h>		// Necessary for the "random" function to work
#include <time.h>		// Necessary for the "time" library functions to work
#include <math.h>
#include <complex.h>		// Necessary for all complex-number functions

int init_010 (void);
int fault_010 (void);
int sim_010 (void);

int
circuit_010 (void)
{
  clear_screen ();

  init_010 ();			// Initialize component values
  fault_010 ();			// Randomly choose a fault
  sim_010 ();			// Simulate circuit

  // Get the current UNIX system time as the timestamp that the user
  // began troubleshooting
  steplog[0].time = (int) (time (NULL));

  while (mistake == 0 && conclusion == -1)	// Loop runs until a mistake or conclusion is made
    {
      print_components (1);	// 1 = Print component nominal values and switch/jumper statuses

      print_measurements ();

      user_test ();

      sim_010 ();

      clear_screen ();

    }

  return 0;
}




int
init_010 (void)
{

  int n;

  tp = 4;			// 2 test points in circuit in addition to TP 0

  // Here we use the "health" integer variable of element 0 for each type
  // of circuit component as a count for how many of these components will
  // be in the circuit, since no circuit ever contains a "component 0" 
  // (e.g. resistor R0 or switch S0).  For jumpers we use the "status"
  // element (i.e. j[0].s).
  //
  // Following the "blank_slate()" function, all health and status values
  // should be set to zero, so all we must do here is specify components 
  // that are in the circuit (i.e. we don't have to declare what IS NOT in 
  // the circuit, but only what IS).
  acv[0].h = 1;
  r[0].h = 1;
  pot[0].h = 1;
  c[0].h = 1;
  l[0].h = 1;

  // We need to re-initialize typical capacitor and inductor values to
  // ensure we don't get unrealistic combinations in our filter circuit
  // Initializing capacitors
  c[1].h = 0;			// All capacitor statuses are healthy (0)
  c[1].alt[0] = 0.001e-6;	// First nominal capacitance value (Farads)
  c[1].alt[1] = 0.002e-6;
  c[1].alt[2] = 0.005e-6;
  c[1].alt[3] = 0.003e-6;
  c[1].alt[4] = 0.002e-6;
  c[1].alt[5] = 0.001e-6;
  c[1].alt[6] = 0.002e-6;
  c[1].alt[7] = 0.001e-6;
  c[1].alt[8] = 0.003e-6;
  c[1].alt[9] = 0.001e-6;
  c[1].selected = c[1].alt[0];	// Set selected nominal capacitance to first value
  c[1].tolerance = 5.0;		// +/- 15% capacitance tolerance from nominal value
  c[1].c = random_component (c[1].selected, c[1].tolerance);	// Set actual capacitance to selected value with tolerance
  c[1].esr[0] = 1;		// Set Equivalent Series Resistance value when healthy
  c[1].esr[1] = 9e8;		// Set Equivalent Series Resistance value when failed open
  c[1].esr[2] = 1e-2;		// Set Equivalent Series Resistance value when failed shorted
  c[1].epr[0] = 9e4;		// Set Equivalent Parallel Resistance value when healthy
  c[1].epr[1] = 9e8;		// Set Equivalent Parallel Resistance value when failed open
  c[1].epr[2] = 1e-2;		// Set Equivalent Parallel Resistance value when failed shorted

  // Initializing inductors
  l[1].h = 0;			// All inductor statuses are healthy (0)
  l[1].alt[0] = 3.3e-3;		// First nominal inductance value (Henrys)
  l[1].alt[1] = 4.7e-3;
  l[1].alt[2] = 3.9e-3;
  l[1].alt[3] = 2.5e-3;
  l[1].alt[4] = 20e-3;
  l[1].alt[5] = 33e-3;
  l[1].alt[6] = 100e-3;
  l[1].alt[7] = 50e-3;
  l[1].alt[8] = 27e-3;
  l[1].alt[9] = 7e-3;
  l[1].selected = l[1].alt[0];	// Set selected nominal inductance to first value
  l[1].tolerance = 5.0;		// +/- 5% inductance tolerance from nominal value
  l[1].l = random_component (l[1].selected, l[1].tolerance);	// Set actual inductance to selected value with tolerance
  l[1].esr[0] = 100;		// Set Equivalent Series Resistance value when healthy
  l[1].esr[1] = 9e8;		// Set Equivalent Series Resistance value when failed open
  l[1].esr[2] = 1e-2;		// Set Equivalent Series Resistance value when failed shorted

  acv[1].selected = acv[1].alt[random_whole (9)];	// Randomly selecting nominal source voltage value
  acv[1].v[0] = random_component (acv[1].selected, acv[1].tolerance);	// Setting actual AC source voltage 

  r[1].tolerance = random_component (10.0, 50.0);	// Randomly sets R1's resistance tolerance between +/-5% and +/-15%
  r[1].selected = r[1].alt[random_whole (9)];
  r[1].r[0] = random_component (r[1].selected, r[1].tolerance);

  // C1 and L1 need to be well-matched in order to avoid unreasonable 
  // reactance values at resonance.  So, instead of individually
  // randomizing C1 and L1, we choose one random value (n) to select
  // pairs of C1 and L1 values known to work well.
  n = random_whole (9);

  c[1].tolerance = random_component (5.0, 50.0);	// Randomly sets C1's capacitance tolerance between +/-2.5% and +/-7.5%
  c[1].selected = c[1].alt[n];
  c[1].c = random_component (c[1].selected, c[1].tolerance);

  l[1].tolerance = random_component (2.0, 50.0);	// Randomly sets L1's inductance tolerance between +/-1% and +/-3%
  l[1].selected = l[1].alt[n];
  l[1].l = random_component (l[1].selected, l[1].tolerance);

  // The potentiometer fulfills no electrical function in this circuit, but really
  // only serves as an adjustment for the AC source's frequency.  Therefore,
  // pot[1].x is really the only variable that matters electrically in this circuit.
  // However, for the sake of providing random numerical values for which to identify
  // which randomized scenario each student receives, we allow the potentiometer's
  // selected resistance value as well as its tolerance be randomized here.
  pot[1].x = random_component (0.5, 90);	// Setting transmitter pot to 50% +/- 45%
  pot[1].tolerance = random_component (10.0, 90.0);	// Randomly sets the potentiometer's total resistance tolerance between +/-1% and +/-19%
  pot[1].selected = pot[1].alt[2 + random_whole (7)];	// Randomly selecting nominal potentiometer value, with alt[2] being the lowest possible value

  vm_red = 2;			// Voltmeter red lead on test point 2
  vm_black = 0;			// Voltmeter black lead on test point 0
  vm_cost = 2;			// Voltmeter costs $2 to move test leads

  am_cost = 5;			// Ammeter costs $5 to move 

  noise = 0.05;			// Meter noise = +/- 0.05%

  return 0;
}




int
fault_010 (void)
{
  int n;
  int max = 7;			// Number of possible faults (excluding 0 = No fault)

  // Initialize fault array
  faults[0] = "No fault";
  faults[1] = "Dead source";
  faults[2] = "Resistor R1 failed open";
  faults[3] = "Resistor R1 failed shorted";
  faults[4] = "Capacitor C1 failed open";
  faults[5] = "Capacitor C1 failed shorted";
  faults[6] = "Inductor L1 failed open";
  faults[7] = "Inductor L1 failed shorted";

  printf
    ("If you haven't already, please open the schematic diagram file \n");
  printf ("`circuit_010.pdf' for reference while troubleshooting \n \n");

  printf
    ("The computer will now randomly select one condition from the following list: \n \n");

  for (n = 0; n <= max; ++n)
    printf ("Condition (%i) = %s \n", n, faults[n]);

  printf
    ("\nYour task will be to determine which one of these conditions exists \n");
  printf
    ("based on virtual tests you will perform on the circuit such as \n");
  printf
    ("adjusting potentiometers and placing meters at various test points. \n \n");

  printf ("The troubleshooting scenario will begin with the voltmeter \n");
  printf
    ("connected between test points TP%i and TP%i and the AC source's \n",
     vm_red, vm_black);
  printf
    ("frequency control (potentiometer 1) set to a random position. \n \n");
  printf ("Please note the potentiometer's nominal value in Ohms is \n");
  printf
    ("irrelevant, as the source's internal frequency responds only to \n");
  printf
    ("wiper position, with 0%% being 0 Hz and 100%% being 100 kHz. \n \n");

  printf ("Please type your name (no spaces, 40 characters maximum) \n");
  printf ("and press the <Enter> key to continue \n");

  scanf ("%40s", username);

  // Discard characters from stdin buffer until a linefeed (LF, or ASCII code 10)
  // is detected.
  while (getchar () != 10)
    {
      // Does nothing!
    }

  clear_screen ();

  fault = random_whole (max);

  // If command-line option "-nofault" or "-fault N" invoked, we override any randomly-chosen
  // fault and set the fault code to the override value!
  if (fault_override >= 0)
    fault = fault_override;

  // Default "par" scores
  par_steps = 4;
  par_cost = 7;
  par_time = 60;		// 60 seconds = 1 minute

  switch (fault)
    {
    case 1:			// Source V1 dead, open
      acv[1].h = 1;
      par_steps = 4;
      par_cost = 9;
      break;

    case 2:			// Resistor R1 failed open
      r[1].h = 1;
      par_steps = 4;
      par_cost = 9;
      break;

    case 3:			// Resistor R1 failed shorted
      r[1].h = 2;
      par_steps = 3;
      par_cost = 4;
      break;

    case 4:			// Capacitor C1 failed open
      c[1].h = 1;
      par_steps = 4;
      par_cost = 3;
      break;

    case 5:			// Capacitor C1 failed shorted
      c[1].h = 2;
      par_steps = 5;
      par_cost = 15;
      break;

    case 6:			// Inductor L1 failed open
      l[1].h = 1;
      par_steps = 4;
      par_cost = 3;
      break;

    case 7:			// Inductor L1 failed shorted
      l[1].h = 2;
      par_steps = 5;
      par_cost = 15;
      break;
    }

  // Calculating "limits" for steps taken, testing budget, and time
  if (fault_override == -1)
    {
      limit_steps = 4 * par_steps;
      limit_cost = 4 * par_cost;
      limit_time = 4 * par_time;
    }

  else
    {
      limit_steps = 999999;
      limit_cost = 999999;
      limit_time = 999999;
    }

  return 0;
}





int
sim_010 (void)
{
  int n, m;

  // Here we call the update_pots() function to update the resistance
  // value for each portion of each potentiometer, based on total
  // resistance (r_total) and wiper position (x).
  update_pots ();

  // Declare shorthand variables
  float f, r1, c1, l1, epr_c, esr_c, epr_l, esr_l;
  float complex zc, zl, z_ctotal, z_ltotal, z_tank, zt;
  float complex v_src, i_src;

  // Initialize shorthand variables
  v_src = acv[1].v[acv[1].h];
  r1 = r[1].r[r[1].h];
  c1 = c[1].c;
  l1 = l[1].l;
  esr_c = c[1].esr[c[1].h];
  epr_c = c[1].epr[c[1].h];
  esr_l = l[1].esr[l[1].h];
  epr_l = l[1].epr[l[1].h];

  // Establich source frequency (0 to 100 kHz) based on pot per-unit setting
  f = pot[1].x * 100e3;
  acv[1].f = f;

  // Set a lower bound for frequency so we don't get 
  // divide-by-zero errors when calculating Z_C
  if (f < 1e-3)
    f = 1e-3;

  // Calculate capacitive and inductive impedances as phasor (complex)
  // quantities.  The complex exponential cexp(j * pi/2) is equal to +j
  // while the complex exponential cexp(j * pi*3/2) is equal to -j
  zl = cexp (I * M_PI / 2) * 2 * M_PI * f * l1;
  zc = cexp (I * M_PI * 3 / 2) * (1 / (2 * M_PI * f * c1));

  // Begin circuit analysis
  z_ltotal = 1 / ((1 / epr_l) + (1 / (zl + esr_l)));
  z_ctotal = 1 / ((1 / epr_c) + (1 / (zc + esr_c)));
  z_tank = 1 / ((1 / z_ltotal) + (1 / z_ctotal));
  zt = r1 + z_tank;

  i_src = v_src / zt;
  i_tp[1] = cabs (i_src);
  i_tp[2] = 0.0;
  i_tp[3] = cabs (i_src * z_tank / z_ctotal);
  i_tp[4] = cabs (i_src * z_tank / z_ltotal);

  // Calculate all test point voltages with respect to ground (TP 0)
  v_tp[0][0] = 0.0;		// TP 0 (ground) is 0 Volts by definition . . .
  v_tp[1][0] = cabs (v_src);
  v_tp[2][0] = cabs (v_src - (i_src * r1));
  v_tp[3][0] = v_tp[2][0];
  v_tp[4][0] = v_tp[2][0];

  // Calculate all test point-pair voltages
  for (n = 0; n < COUNT; ++n)
    {
      for (m = 0; m < COUNT; ++m)
	v_tp[n][m] = v_tp[n][0] - v_tp[m][0];
    }

  return 0;
}
